//############################################################################
//								Propeller closck
//								Author: Thomas Schattat
//                              Copyright (C) 2008
//############################################################################
	
//#### INCLUDES ####
#include "main.h"
#include <string.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <util/delay.h>
//############## CONSTANTS ##############

//############## VARIABLES ##############
volatile unsigned char  sec_fract=30, second=30, minute=58, hour=10, display[4], Timer_Rest, Timer_NoRest;
volatile signed int Position=0, PosMin, PosHour, Delay;
volatile unsigned int Rest;
volatile long unsigned int average;
volatile bool too_slow = false, show = false;

void Send_String (char*);

//#### MAIN ####
int main (void)
{
	TCCR0 |= (1<<CS01)|(1<<WGM01);			//Prescaling of 8 timer0 (14,31MHz / 8) and CTC mode.
	OCR0=preload0;
	TCNT0 = 0;								//8 bit Timer0
	TCCR1B |= (1<<CS11);					//Prescaling of 8 timer1 (14,31MHz / 8).
	TCCR2 |= (1<<CS21)|(1<<WGM21);			//Prescaling of 8 timer2 (14,31MHz / 8) and CTC mode.
	TIMSK |= (1<<OCIE2)|(1<<OCIE0)|(1<<TOIE1);	//Set timer  interrupts.
	TCNT2 = 0;								//Set initial value for Timer2
	TCNT1 = 0;								//16 bit Timer preload	
	Delay = 7155;
	
	MCUCR |= (1<<ISC01);				//ext IRQ falling edge
	GICR |= (1<<INT0);					//enable ext.IRQ0
	SREG |= 0x80;						//Global Interrupt enable
	
	DDRA = 0xFF;						//all ports are outputs
	DDRB = 0xFF;
	DDRC = 0xFF;
	DDRD = 0xFA;						//keep PD0 as input for serial data and PD2 as INT0
	
	//configure SPI for shift registers...
	SPSR |=(1<<SPI2X); 
	SPCR = (1<<SPE)|(1<<MSTR); 		// Enable SPI, Master, set clock rate fck/2
	
	display[3]=0;
	display[2]=0;
	display[1]=0;
	display[0]=0xF0;						//all LEDs off, 128=R, 64=G, 32=B
	
	while(1)
	{

	}
}

//#### TIMER 2 INTERRUPT, has highest IRQ priority!!  ####
ISR(TIMER2_COMP_vect)						//Timer2 Interrupt
{
	if (!Rest--)
		OCR2 = Timer_NoRest;
	 
	if (Position > max_degree)
		TCNT2=0;							//external interrupt expected soon
	
	PORTB &= 255-out_en;					// LEDs off before shift
	//if (show)
	{
	SPDR = display[0];						// send out data
	while(!(SPSR & (1<<SPIF))) ;			// wait until sent
	SPDR = display[1];						// send out data
	while(!(SPSR & (1<<SPIF))) ;			// wait until sent
	SPDR = display[2];						// send out data
	while(!(SPSR & (1<<SPIF))) ;			// wait until sent
	SPDR = display[3];						// send out data
	while(!(SPSR & (1<<SPIF))) ;			// wait until sent
	PORTB |= out_en;						// LEDs on after shift!!
	
	//calculate next display..
	 display[0]=0xF0;
	 display[1]=0;
	 display[2]=0;
	 display[3]=0;							//all off first except outer ring
	 if ((Position < (max_degree/60)*second))
		display[0]-=128;
	 if (!(Position%(max_degree/4)))
		display[0]|=15;
	 if (!(Position%(max_degree/12)))
		display[0]|=14;
	 if (!(Position%(max_degree/60)))
		display[0]|=8;	
		
	 //###hands of clock here#######
	 if (Position==(max_degree+PosMin-2)%max_degree)	//all including Pos 238
		display[1]|=16;	
	 if (Position==PosMin-1)							//all except Pos 239
		display[1]|=48;
	 if (Position==PosMin)
		{
		display[1]|=127;
		display[2]|=255;
		display[3]|=255;
		}
	 if (Position==PosMin+2)
		display[1]|=16;	
	 if (Position==PosMin+1)
		display[1]|=48;	
	 if (Position==PosHour-2)
		display[2]|=32;	
	 if (Position==PosHour-1)
		display[2]|=96;
	 if (Position==PosHour)
		{
		display[3]=255;
		display[2]=255;
		}
	 if (Position==PosHour+2)
		display[2]|=32;	
	 if (Position==PosHour+1)
		display[2]|=96;
	}
	Position++;
	//display[3]=LED_on;
}

//#### External Interrupt from magnet at 12 o'clock####
ISR(SIG_INTERRUPT0)							//highest PRIO of all
{
    unsigned int duration, TimerInterval;
	cli();
	duration= TCNT1;							//remember last counter value
	TCNT1 = 0;
	TimerInterval = duration /max_degree;
	Timer_NoRest = ((TimerInterval-1) & 0x00FF);
	Timer_Rest = 1+ Timer_NoRest;
	OCR2 = Timer_Rest;
	Rest = duration % max_degree;
	
	TCNT2 = OCR2-1;								// Trigger Timer interupt shortly afterwards
	
	display[0]=0xF0;							// 12 o'clock mark
	display[1]=0;
	display[2]=0;
	display[3]=0;
	
	//#########This is position 239 !! ##########
	if (!PosMin)
		display[1]|=48;
	if (!PosHour)
		display[2]|=96;
	//###########################################
	Position=0;
	if (!too_slow)
		show = true;
	else
		show= false;
	too_slow= false;
}

//#### TIMER 1 INTERRUPT ####
ISR(TIMER1_OVF_vect)						//Timer1 Interrupt should never fire
{ 
 too_slow = true;		
}

//#############count clock here ################################
ISR(TIMER0_COMP_vect)						//Timer0 Interrupt
{

 sei();										//allow nested iRQ
 if (!--Delay)
 {
  Delay = 7155;
  if (++second > 59)
   {
    second = 0;
	if (++minute > 59)
	{
	 minute=0;
	 if (++hour > 23)
	   hour=0;
	}
   }
   PosMin=(max_degree/60)*minute;
   PosHour=(5*(hour%12)+minute/12)*(max_degree/60);
 } 
}